//
//  Factorise.m
//  Factorisation class: factorises integers read from standard input
//
//  Created by Rene Hexel on 20/10/06.
//  Copyright 2006-2008 Rene Hexel. All rights reserved.
//

#import "Factorise.h"
#include <stdio.h>

@implementation Factorise
/**
 * Read factorisation values from standard input and
 * print results on standard output
 */
- doIt
{
	// get a file handle for standard input (console input)
	NSFileHandle *in = [NSFileHandle fileHandleWithStandardInput];
	NSData *in_data;		// the data to read

	while ((in_data = [in availableData]) != nil)
	{
		NSAutoreleasePool *pool = [NSAutoreleasePool new];

		NSString *input = [ [NSString alloc]
				    initWithData: in_data
					encoding: NSUTF8StringEncoding ];
		
		NSScanner *scanner = [NSScanner scannerWithString: input];

		while (![scanner isAtEnd])
		{
			int n;		// the integer to factorise

			if (![scanner scanInt: &n])
				break;	// not an integer that we read?
			
			NSArray *factors = [self factorise: n];
			[self printFactors: factors forInt: n];
		}

		[input release];	// release the alloc'ed string
		[pool release];		// release temporary variables
	}
	
	return self;
}


/**
 * Factorise a given integer into an array
 */
- (NSArray *) factorise: (int) n
{
	// create an array for factors
	NSMutableArray *factors = [NSMutableArray new];

	int c;
	for (c = 2;  n > 1 && c*c <= n;  c++)
	{
		while (n % c == 0)
		{
			NSNumber *factor = [NSNumber numberWithInt: c];

			[factors addObject: factor];

			n /= c;		// same as:  n = n / c
		}
	}
	
	if (n > 1)
		[factors addObject: [NSNumber numberWithInt: n] ];

	return [factors autorelease];
}


/**
 * print the factors in the array using printf()
 */
- printFactors: (NSArray *) factors
	forInt: (int) n
{
	NSAutoreleasePool *pool = [NSAutoreleasePool new];
	NSEnumerator *e = [factors objectEnumerator];
	NSNumber *factor = [e nextObject];
	
	printf("%d = ", n);
	
	while (factor)
	{
		printf("%d", [factor intValue]);
		
		factor = [e nextObject];
		
		if (factor)
			printf(" . ");
	}
	printf("\n");

	[pool release];		// releases the objectEnumerator

	return self;
}

@end
